#include <Wire.h>
#include <avr/sleep.h>
#include <BMP085.h>

/*arduino pro mini
=====================================================================
RUNNING (pwr supply)                                     : 19   mA
RUNNING (3.3V direct)                                    : 17   mA
SLEEP_MODE_PWR_DOWN                                      : 15.0 mA
=====================================================================

atmega328 (arduino pro mini with pwr. supply & pwr. led removed)
=====================================================================
RUNNING                                                  : 15.5  mA
SLEEP_MODE_IDLE                                          : 15    mA
SLEEP_MODE_ADC                                           : 6.5   mA
SLEEP_MODE_PWR_SAVE                                      : 1.62  mA
SLEEP_MODE_EXT_STANDBY                                   : 1.62  mA
SLEEP_MODE_STANDBY                                       : 0.84  mA
SLEEP_MODE_PWR_DOWN                                      : 360   uA
SLEEP_MODE_PWR_DOWN (BROWN OUT disabled)                 : 335   uA
SLEEP_MODE_PWR_DOWN (ADC disabled)                       : 0.335 uA
---------------------------------------------------------------------
real board consumption (external int set up - pwr. down) : 0.3   mA
real board consumption (active incl. 1 led for 50ms)     : 3.7   mA
====================================================================*/

//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

/// Bit handling macros
#define SETBIT(x,y)  (x |= (y))    // set bit y in byte x
#define CLRBIT(x,y)  (x &= (~(y))) // clear bit y in byte x
/// Bit handling macros end

#define smillis() ((long)millis()) /// millis() rollover handling

/// Pin setup
#define RESET_PIN    7
#define WAKE_UP_INT  0 // external interrupt on arduino pin D2
#define WAKE_UP_PIN  2 // external interrupt on arduino pin D2
#define LED_PIN      13
/// Pin setup end

/// Software Reset
void reset_device() {
  pinMode(RESET_PIN, OUTPUT);
  digitalWrite(RESET_PIN, HIGH);
}
/// Software Reset end

/// Wake-up from sleep
void wake() {
  reset_device();
}
/// Wake-up from sleep end

/// initialize the sensors
BMP085 sensor;
/// initialize the sensors end

//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
void setup() {
  Serial.begin(9600);       // debug/programming/kindle communication
  Serial.flush();
  pinMode(LED_PIN, OUTPUT); // make Led pin ouput
  
  sensor.begin();           // start the sensor
  
  digitalWrite(LED_PIN, HIGH); // turn on led - activity incoming
  
  long cur_smillis = smillis();
  while(smillis() - cur_smillis < 50) {
    processTerminalData();
  }
} // Setup end

void loop() {
  delay(5);
  
  pinMode (0,  OUTPUT); digitalWrite (0,  LOW);
  pinMode (1,  OUTPUT); digitalWrite (1,  LOW);
  //pinMode (2,  OUTPUT); digitalWrite (2,  LOW); // external interrupt - don't touch the pin state!!!
  pinMode (3,  OUTPUT); digitalWrite (3,  LOW);
  pinMode (4,  OUTPUT); digitalWrite (4,  LOW);
  pinMode (5,  OUTPUT); digitalWrite (5,  LOW);
  pinMode (6,  OUTPUT); digitalWrite (6,  LOW);
  pinMode (7,  OUTPUT); digitalWrite (7,  LOW);
  pinMode (8,  OUTPUT); digitalWrite (8,  LOW);
  pinMode (9,  OUTPUT); digitalWrite (9,  LOW);
  pinMode (10, OUTPUT); digitalWrite (10, LOW);
  pinMode (11, OUTPUT); digitalWrite (11, LOW);
  pinMode (12, OUTPUT); digitalWrite (12, LOW);
  pinMode (13, OUTPUT); digitalWrite (13, LOW);
  pinMode (A0, OUTPUT); digitalWrite (A0, LOW);
  pinMode (A1, OUTPUT); digitalWrite (A1, LOW);
  pinMode (A2, OUTPUT); digitalWrite (A2, LOW);
  pinMode (A3, OUTPUT); digitalWrite (A3, LOW);
  pinMode (A4, OUTPUT); digitalWrite (A4, LOW);
  pinMode (A5, OUTPUT); digitalWrite (A5, LOW);
  
  CLRBIT(ADCSRA, (1<<ADEN)); // disable ADC
  
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);
  sleep_enable();
  
  // turn off brown-out
  SETBIT(MCUCR, (1<<BODSE) + (1<<BODS)); 
  SETBIT(MCUCR, (1<<BODS));
  CLRBIT(MCUCR, (1<<BODSE));
  
  attachInterrupt(WAKE_UP_INT, wake, LOW); // wake() will be called when pin D2 goes low
  
  sleep_mode();
} // Loop end

//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
void processTerminalData() {
  char cmdbuf[20];
  int i = 0;
  bool done = false;
  
  long cur_smillis = smillis();
  while(1) {
    if(Serial.available() > 0) {
      cmdbuf[i] = Serial.read();
      
      if(cmdbuf[i] == '\r') i = 0;
      
      if(cmdbuf[i] == '%') {
        if(cmdbuf[i-2] == '*' && cmdbuf[i-1] == '@') {
          Serial.print(F("{\"stat\":[{\"id\":\"temp\",\"val\":\""));
          Serial.print(sensor.readTemperature());              // temperature [°C]
          Serial.print(F("\"},{\"id\":\"pres\",\"val\":\""));
          Serial.print((float) sensor.readPressure()*0.0075);  // pressure [mmHg]
          Serial.print(F("\"},{\"id\":\"alt\",\"val\":\""));
          Serial.print(sensor.readAltitude(101110));           // altitude [m] (based on pressure at sea level in "Gura Portitei")
          Serial.print(F("\"}]}"));
          Serial.print("\r");
          
          done = true;
        }
        Serial.flush();
        i = 0;
      }
      
      i++;
    }
    
    if( ( i    == sizeof(cmdbuf)      ) ||
        ( done == true                ) ||
        ( smillis() - cur_smillis > 5 )
      ) {
      i = 0;
      
      return;
    }
  }
} // processTerminalData end
